@@ -22,6 +22,7 @@
 #ifdef SJOIN
 E void load_sjoin_mem();
 E void do_sjoin(char *source, int ac, char **av);
+E void checkmass_shouldop(User *user, const char *chan, const char *modestr);
 #endif
 E int has_chan_status(User *u, Channel *c, int status);
 E void remove_chan_status(User *u, Channel *c, int status);
@@ -39,6 +40,7 @@ E void kick_user (const char *who, const char *chan, const char *nick, const cha
 E int channel_form();
 E char *chanmode[];
 E char *filter_modes (char *chan, char *on, char *off);
+E void wipemask (Channel *c, char *mask, char *who);
 /**** chanserv.c ****/
 E int isinban(Channel *c, User *u);
 E int check_kick(User *user, const char *chan);
@@ -48,7 +50,7 @@ E void check_welcome(User *user, const char *chan);
 E void listchans(int count_only, const char *chan);
 E void get_chanserv_stats(long *nrec, long *memuse);
 
-E void chanserv(const char *source, char *buf);
+E void chanserv(User *u, const char *source, char *buf);
 E void load_cs_dbase(void);
 E void save_cs_dbase(void);
 E void check_modes(const char *chan, const char *source);
@@ -64,8 +66,8 @@ E int check_hopguard(User *user, const char *chan, int newchan);
 E int check_valid_hop(User *user, const char *chan, int newchan);
 #endif
 E int check_akick(User *user, const char *chan);
-E void record_topic(const char *chan);
-E void restore_topic(const char *chan);
+E void record_topic(const char *chan, Channel *c);
+E void restore_topic(const char *chan, Channel *c);
 E int check_topiclock(const char *source, const char *chan);
 E void check_temp_os(User *u);
 E void expire_chans(void);
@@ -150,7 +152,7 @@ E char *sstrdup(const char *s);
 
 E void get_memoserv_stats(long *nrec, long *memuse);
 
-E void memoserv(const char *source, char *buf);
+E void memoserv(User *u, const char *source, char *buf);
 E void load_ms_dbase(void);
 E void save_ms_dbase(void);
 E void check_memos(const char *nick);
@@ -231,7 +233,7 @@ E void get_nickserv_stats(long *nrec, long *memuse);
 E int is_on_access(User *u, NickInfo *ni);
 E void do_set_ossop (const char *source);
 E void do_set_oshop (const char *source);
-E void nickserv(const char *source, char *buf);
+E void nickserv(User *u, const char *source, char *buf);
 E void load_ns_dbase(void);
 E void save_ns_dbase(void);
 E int validate_user(User *u);
@@ -402,19 +404,19 @@ E void load_chtml_db(void);
 E void load_wordgame_db(void);
 E void save_bot_db(void);
 E void do_adlhost(const char *source);
-E void agent(const char *source, char *buf);
+E void agent(const char *source, char *buf, User *u);
 #ifndef NONSUPER
 E void servicesX(const char *source, char *buf);
 E void servicesW(const char *source, char *buf);
 E void massserv(const char *source, char *buf);
 #endif
-E void HostServ(const char *source, char *buf);
+E void HostServ(User *u, const char *source, char *buf);
 E void sbotserv(const char *source, char *buf);
 E void SpamCheck(const char *source, char *buf);
 E int check_password(const char *plaintext, const char *password);
 E void botserv(const char *source, const char *thebot, char *buf);
 E int do_match_wild(const char *pattern, const char *str, int docase);
-E void bmsgchan(const char *source, char *chan, char *buf);
+E void bmsgchan(const char *source, char *chan, char *buf, User *u);
 E int  is_validnick(char *nick);
 E void make_channel_html(void);
 E void cleanmemory(const char *curversion);
@@ -443,7 +445,9 @@ E void load_ignore(void);
 E void create_dummy_logs(void);
 E void open_slog(void);
 E void listmemos(const char *nick, const char *pass);
-
+E void addtban(Channel *c, char *mask, int mins, char *who);
+E void checktban(Channel *c);
+E void deltban(Channel *c, char *mask);
 /**** log.c ****/
 
 E FILE *logfile;
@@ -471,7 +475,7 @@ E void open_log(void);
 #endif
 /***** RootServ *****/
 
-E void rootserv(const char *source, char *buf);
+E void rootserv(User *u, const char *source, char *buf);
 
 /***** conf.c *****/
 E int   BSKeepData;
@@ -492,7 +496,8 @@ E char *SERVICES_HOST;
 E char *NETWORK_NAME;
 E char *SERVICES_MASTER;
 E char *server_num;
-
+E char *CLOAK_KEY;
+E int istoken;
 E char *NICKSERV_NAME;
 E char *CHANSERV_NAME;
 E char *MEMOSERV_NAME;
@@ -675,15 +680,23 @@ extern void sendmotd(const char *source);
 /************* webserv.c *********************/
 #ifdef WEBSERVICE
 E char *s_WebServ;
-E void webserv (const char *source, char *buf);
-E void do_nsregister (const char *source);
-E void do_csregister (const char *source);
-E void do_nsidentify (const char *source);
-E void do_csidentify (const char *source);
+E void webserv (User *u, const char *source, char *buf);
+E void do_nsregister (User *u, const char *source);
+E void do_csregister (User *u, const char *source);
+E void do_nsidentify (User *u, const char *source);
+E void do_csidentify (User *u, const char *source);
 #endif
 /************* adminserv.c *******************/
 E int is_inolevel(const char *source, int alevel);
 E int is_inilevel(const char *source, int alevel);
 E int getzone();
+E void curzone(const char *source);
 E int is_adminlevel(const char *source);
+E void ns_gnick(User *u);
+/************** email.c **********************/
+void send_auth(char *who, char *nick, char *email, char *codes);
+void send_expirex(char *who, NickInfo *ni);
+void send_chanmemo(char *who, NickInfo *ni, char *chan, char *text);
+void send_amemo(char *who, NickInfo *ni, char *text);
+void sendit(const char *who, char *nick, char *email);
 #endif	/* EXTERN_H */
