@@ -28,6 +28,8 @@ static User *new_user(const char *nick)
     NickInfo *ni;
 
     user = scalloc(sizeof(User), 1);
+    if (!nick)
+        nick = "";
     strscpy(user->nick, nick, NICKMAX);
     user->floodlev = 0;
     user->next = userlist;
@@ -45,10 +47,6 @@ static User *new_user(const char *nick)
     if (is_tokn(34))
 	    m_sendmotd(nick);
 
-#ifndef BAHAMUT
-    send_cmd(NULL, "USERHOST %s", nick);
-#endif
-
     if (is_tokn(20)) {
 	ni = findnick(nick);
 	if (!ni) {
@@ -78,10 +76,6 @@ void delete_user(User *user)
     struct u_chaninfolist *ci, *ci2;
     int i;
 
-    if(!user) {
-	wallops(SERVER_NAME, "Look like trying to free non exist user.");
-	return;
-    }
     usercnt--;
     if (user->mode & UMODE_O)
 	opcnt--;
@@ -286,24 +280,34 @@ void do_nick(const char *source, int ac, char **av)
 #else
 	user->username = sstrdup(av[3]);
 	user->realhost = sstrdup(av[4]);
-	user->host = sstrdup(av[4]);
 	user->server = sstrdup(av[5]);
 #ifdef NICKV2
         av[1] = av[7];
 	user->realname = sstrdup(av[9]);
+	user->host = sstrdup(av[8]);
         do_umode(av[0], 2, av);
 #else
+
+#ifdef CR
+	user->realname = sstrdup(av[7]);
+#else
 	user->realname = sstrdup(av[6]);
 #endif
+
+	user->host = sstrdup(av[4]);
+#endif /* nickv2 */
+
 #endif
 	user->games = 0;
+
 	if (!stricmp(user->server, WEBTVSERV)) {
 		user->iswebtv = 1;
 	} else {
 		user->iswebtv = 0;
 	}
+
 	/* Check to see if it looks like clones. */
-        if (clone_detect == 1)
+        if (is_tokn(5))
            check_clones(user);
 
         /* Check to see if realname setting matches against warning list */
@@ -316,28 +320,15 @@ void do_nick(const char *source, int ac, char **av)
 	do_autojoin(user->nick, user->host, user->username);
 
     } else {
+
 	/* An old user changing nicks. */
-	NickInfo *ni, *hni;
 	user = finduser(source);
+
 	if (!user) {
-	    log("user: NICK from nonexistent nick %s: %s", source,
-							merge_args(ac, av));
-		    return;
+	    log("user: NICK from nonexistent nick %s: %s", source, merge_args(ac, av));
+	    return;
 	}
 
-        ni = findnick(source);
-        hni = nickhost(ni);
-
-        if (hni && (!(hni->flags & NI_VERBOTEN)) &&
-            (((hni->flags & NI_RECOGNIZED) && (!hni->flags & NI_SECURE)) ||
-            (hni->flags & NI_IDENTIFIED))) {
-            hni->last_seen = CTime;
-            if (hni->last_usermask)
-                free(hni->last_usermask);
-            hni->last_usermask = smalloc(strlen(user->username)+strlen(user->host)+2);
-            sprintf(hni->last_usermask, "%s@%s", user->username, user->host);
-        }
-
 	if (debug)
 	    log("debug: %s changed nick to %s", source, av[0]);
 
@@ -346,20 +337,16 @@ void do_nick(const char *source, int ac, char **av)
 	change_user_nick(user, av[0]);
 
 	user->signon = atol(av[1]);
-/*
-	if (strlen(av[0]) >= NICKMAX) {
-	    send_cmd(SERVER_NAME, "KILL %s :Invalide nick for services.", av[0]);
-	    do_kill(SERVER_NAME, 3, av);
-	    return;
-	}
-*/
+
     }
+
     user->my_signon = CTime;
 
     if (validate_user(user)) {
 	check_memos(user->nick);
 	check_auth(user->nick);
     }
+
 }
 
 /*************************************************************************/
@@ -388,19 +375,19 @@ void do_join(const char *source, int ac, char **av)
 		if (debug) {
 			log("debug: %s joins %s", source, s);
 	}
-    if (*s == '0') {
-         c = user->chans;
-         while (c)
-         {
-             c2 = c->next;
-             chan_deluser(user, c->chan);
-             free (c);
-             c = c2;
-         }
-         user->chans = NULL;
-         continue;
-    }
 
+	if (*s == '0') {
+        	 c = user->chans;
+	         while (c)
+        	 {
+	             c2 = c->next;
+        	     chan_deluser(user, c->chan);
+	             free (c);
+        	     c = c2;
+	         }
+        	 user->chans = NULL;
+	         continue;
+	}
 
         /* Make sure check_kick comes before chan_adduser, so banned users
 	 * don't get to see things like channel keys. */
@@ -419,6 +406,8 @@ void do_join(const char *source, int ac, char **av)
 	    user->chans->prev = c;
 	user->chans = c;
 	c->chan = findchan(s);
+
+	checkmassop(user, s);
     }
 }
 
@@ -570,7 +559,7 @@ void do_umode(const char *source, int ac, char **av)
 				if  (is_oper_cando(user->nick, 1)) {
 					user->mode |= UMODE_N;
 				} else {
-					send_cmd(s_NickServ, "SVSMODE %s -N 0", user->nick);
+					send_cmd(s_NickServ, "%s %s -N 0", istoken?TOK_SVSMODE:MSG_SVSMODE, user->nick);
 				}
 			} else {
 				user->mode |= UMODE_N;
@@ -593,7 +582,7 @@ void do_umode(const char *source, int ac, char **av)
 			user->mode |= UMODE_O;
 #ifdef SVSJOIN
 			if (is_tokn(21))
-			    send_cmd(s_Agent, "%s %s %s", SVSJOIN, user->nick, tokn[17]);
+				send_cmd(s_Agent, "%s %s %s", istoken?TOK_SVSJOIN:MSG_SVSJOIN, user->nick, tokn[17]);
 #endif
 	      		do_operlogonmsg(user->nick);
 
@@ -601,7 +590,7 @@ void do_umode(const char *source, int ac, char **av)
 
 		    } else {
 			send_cmd(s_OperServ, "PRIVMSG #Snoop :Failed oper for %s", user->nick);
-			send_cmd(NULL, "SVSMODE %s -oaA", user->nick);
+			send_cmd(NULL, "%s %s -oaA", istoken?TOK_SVSMODE:MSG_SVSMODE, user->nick);
 			notice(s_OperServ, user->nick, "Sorry, You have the Abusive user mode set, You cannot oper.");
 		    }
 
@@ -657,7 +646,7 @@ void do_quit(const char *source, int ac, char **av)
 		}
 	      }
     }
-	    delete_user(user);
+    delete_user(user);
 }
 
 /*************************************************************************/
@@ -1063,7 +1052,7 @@ void alljoin(char *chan)
 {
     User *user;
         for (user = userlist; user; user = user->next) {
-        	send_cmd(s_MassServ, "SAJOIN %s %s",user->nick,chan);
+        	send_cmd(s_MassServ, "%s %s %s", istoken?TOK_SAJOIN:MSG_SAJOIN, user->nick,chan);
 		notice(s_MassServ, user->nick, "You are invited to join channel %s by Network Admin",chan);
 	}
 }
